@@ -1,6 +1,6 @@
 /*
-   mod_ruid2 0.9.4
-   Copyright (C) 2011 Monshouwer Internet Diensten
+   mod_ruid2 0.9.7
+   Copyright (C) 2009-2012 Monshouwer Internet Diensten
 
    Author: Kees Monshouwer
 
@@ -26,14 +26,17 @@
    - /usr/apache/bin/apxs -a -i -l cap -c mod_ruid2.c
 */
 
+#include "ap_release.h"
+
+/* define CORE_PRIVATE for apache < 2.4 */
+#if AP_SERVER_MAJORVERSION_NUMBER == 2 && AP_SERVER_MINORVERSION_NUMBER < 4
 #define CORE_PRIVATE
+#endif
 
 #include "apr_strings.h"
 #include "apr_md5.h"
 #include "apr_file_info.h"
-#include "ap_config.h"
-#include "httpd.h"
-#include "http_config.h"
+#include "unixd.h"
 #include "http_core.h"
 #include "http_log.h"
 #include "http_protocol.h"
@@ -45,34 +48,41 @@
 #include <sys/capability.h>
 
 #define MODULE_NAME		"mod_ruid2"
-#define MODULE_VERSION		"0.9.4"
+#define MODULE_VERSION		"0.9.7"
 
-#define RUID_DEFAULT_UID	48
-#define RUID_DEFAULT_GID	48
 #define RUID_MIN_UID		100
 #define RUID_MIN_GID		100
 
-#define RUID_MAXGROUPS		4
+#define RUID_MAXGROUPS		8
 
-#define RUID_MODE_STAT		0
-#define RUID_MODE_CONF		1
+#define RUID_MODE_CONF		0
+#define RUID_MODE_STAT		1
 #define RUID_MODE_UNDEFINED	2
 
+#define RUID_MODE_STAT_NOT_USED	0
+#define RUID_MODE_STAT_USED	1
+#define RUID_CHROOT_NOT_USED	0
+#define RUID_CHROOT_USED	1
+
 #define RUID_CAP_MODE_DROP	0
 #define RUID_CAP_MODE_KEEP	1
 
+#define NONE			-2
 #define UNSET			-1
 #define SET			1
 
+/* added for apache 2.0 and 2.2 compatibility */
+#if !AP_MODULE_MAGIC_AT_LEAST(20081201,0)
+#define ap_unixd_config unixd_config
+#endif
 
 typedef struct
 {
 	int8_t ruid_mode;
-
 	uid_t ruid_uid;
 	gid_t ruid_gid;
 	gid_t groups[RUID_MAXGROUPS];
-	int8_t groupsnr;
+	int groupsnr;
 } ruid_dir_config_t;
 
 
@@ -82,7 +92,6 @@ typedef struct
 	gid_t default_gid;
 	uid_t min_uid;
 	gid_t min_gid;
-	int8_t stat_used;
 	const char *chroot_dir;
 	const char *document_root;
 } ruid_config_t;
@@ -91,18 +100,31 @@ typedef struct
 module AP_MODULE_DECLARE_DATA ruid2_module;
 
 
-static int coredump, cap_mode, stat_mode, chroot_root;
+static int mode_stat_used	= RUID_MODE_STAT_NOT_USED;
+static int chroot_used		= RUID_CHROOT_NOT_USED;
+
+static int coredump, cap_mode, root_handle;
 static const char *old_root;
 
+static gid_t startup_groups[RUID_MAXGROUPS];
+static int startup_groupsnr;
+
 
 static void *create_dir_config(apr_pool_t *p, char *d)
 {
+	char *dname = d;
 	ruid_dir_config_t *dconf = apr_pcalloc (p, sizeof(*dconf));
 
-	dconf->ruid_mode=RUID_MODE_UNDEFINED;
+	if (dname == NULL) {
+		// Server config
+		dconf->ruid_mode=RUID_MODE_CONF;
+	} else {
+		// Directory config
+		dconf->ruid_mode=RUID_MODE_UNDEFINED;
+	}
 	dconf->ruid_uid=UNSET;
 	dconf->ruid_gid=UNSET;
-	dconf->groupsnr=0;
+	dconf->groupsnr=UNSET;
 
 	return dconf;
 }
@@ -120,18 +142,20 @@ static void *merge_dir_config(apr_pool_t *p, void *base, void *overrides)
 		conf->ruid_mode = child->ruid_mode;
 	}
 	if (conf->ruid_mode == RUID_MODE_STAT) {
-		conf->ruid_uid=RUID_DEFAULT_UID;
-		conf->ruid_gid=RUID_DEFAULT_GID;
-		conf->groupsnr=0;
+		conf->ruid_uid=UNSET;
+		conf->ruid_gid=UNSET;
+		conf->groupsnr=UNSET;
 	} else {
 		conf->ruid_uid = (child->ruid_uid == UNSET) ? parent->ruid_uid : child->ruid_uid;
 		conf->ruid_gid = (child->ruid_gid == UNSET) ? parent->ruid_gid : child->ruid_gid;
-		if (child->groupsnr != 0) {
+		if (child->groupsnr > 0) {
 			memcpy(conf->groups, child->groups, sizeof(child->groups));
 			conf->groupsnr = child->groupsnr;
-		} else {
+		} else if (parent->groupsnr > 0) {
 			memcpy(conf->groups, parent->groups, sizeof(parent->groups));
 			conf->groupsnr = parent->groupsnr;
+		} else {
+			conf->groupsnr = (child->groupsnr == UNSET) ? parent->groupsnr : child->groupsnr;
 		}
 	}
 
@@ -143,11 +167,10 @@ static void *create_config (apr_pool_t *p, server_rec *s)
 {
 	ruid_config_t *conf = apr_palloc (p, sizeof (*conf));
 
-	conf->default_uid=RUID_DEFAULT_UID;
-	conf->default_gid=RUID_DEFAULT_GID;
+	conf->default_uid=ap_unixd_config.user_id;
+	conf->default_gid=ap_unixd_config.group_id;
 	conf->min_uid=RUID_MIN_UID;
 	conf->min_gid=RUID_MIN_GID;
-	conf->stat_used=UNSET;
 	conf->chroot_dir=NULL;
 	conf->document_root=NULL;
 
@@ -158,7 +181,6 @@ static void *create_config (apr_pool_t *p, server_rec *s)
 /* configure option functions */
 static const char *set_mode (cmd_parms *cmd, void *mconfig, const char *arg)
 {
-	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
 	ruid_dir_config_t *dconf = (ruid_dir_config_t *) mconfig;
 	const char *err = ap_check_cmd_context (cmd, NOT_IN_FILES | NOT_IN_LIMIT);
 
@@ -166,70 +188,17 @@ static const char *set_mode (cmd_parms *cmd, void *mconfig, const char *arg)
 		return err;
 	}
 
-	if (strcasecmp(arg,"config")==0) {
-		dconf->ruid_mode=RUID_MODE_CONF;
-	} else {
+	if (strcasecmp(arg,"stat")==0) {
 		dconf->ruid_mode=RUID_MODE_STAT;
-		conf->stat_used=SET;
+		mode_stat_used |= RUID_MODE_STAT_USED;
+	} else {
+		dconf->ruid_mode=RUID_MODE_CONF;
 	}
 
 	return NULL;
 }
 
 
-static const char *set_groups (cmd_parms *cmd, void *mconfig, const char *arg)
-{
-	ruid_dir_config_t *dconf = (ruid_dir_config_t *) mconfig;
-	const char *err = ap_check_cmd_context (cmd, NOT_IN_FILES | NOT_IN_LIMIT);
-
-	if (err != NULL) {
-		return err;
-	}
-
-	if (strcasecmp(arg,"@none")==0) {
-	    dconf->groupsnr=-1;
-	}
-	
-	if (dconf->groupsnr<RUID_MAXGROUPS && dconf->groupsnr>-1) {
-		dconf->groups[dconf->groupsnr++] = ap_gname2id (arg);
-	}
-
-	return NULL;
-}
-
-
-static const char *set_minuidgid (cmd_parms *cmd, void *mconfig, const char *uid, const char *gid)
-{
-	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
-	const char *err = ap_check_cmd_context (cmd, NOT_IN_DIR_LOC_FILE | NOT_IN_LIMIT);
-
-	if (err != NULL) {
-		return err;
-	}
-
-	conf->min_uid = ap_uname2id(uid);
-	conf->min_gid = ap_gname2id(gid);
-
-	return NULL;
-}
-
-
-static const char *set_defuidgid (cmd_parms *cmd, void *mconfig, const char *uid, const char *gid)
-{
-	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
-	const char *err = ap_check_cmd_context (cmd, NOT_IN_DIR_LOC_FILE | NOT_IN_LIMIT);
-
-	if (err != NULL) {
-		return err;
-	}
-
-	conf->default_uid = ap_uname2id(uid);
-	conf->default_gid = ap_gname2id(gid);
-
-	return NULL;
-}
-
-
 static const char *set_uidgid (cmd_parms *cmd, void *mconfig, const char *uid, const char *gid)
 {
 	ruid_dir_config_t *dconf = (ruid_dir_config_t *) mconfig;
@@ -246,6 +215,62 @@ static const char *set_uidgid (cmd_parms *cmd, void *mconfig, const char *uid, c
 }
 
 
+static const char *set_groups (cmd_parms *cmd, void *mconfig, const char *arg)
+{
+	ruid_dir_config_t *dconf = (ruid_dir_config_t *) mconfig;
+	const char *err = ap_check_cmd_context (cmd, NOT_IN_FILES | NOT_IN_LIMIT);
+
+	if (err != NULL) {
+		return err;
+	}
+
+	if (strcasecmp(arg,"@none") == 0) {
+		dconf->groupsnr=NONE;
+	}
+
+	if (dconf->groupsnr == UNSET) {
+		dconf->groupsnr = 0;
+	}
+	if ((dconf->groupsnr < RUID_MAXGROUPS) && (dconf->groupsnr >= 0)) {
+		dconf->groups[dconf->groupsnr++] = ap_gname2id (arg);
+	}
+
+	return NULL;
+}
+
+
+static const char *set_defuidgid (cmd_parms *cmd, void *mconfig, const char *uid, const char *gid)
+{
+	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
+	const char *err = ap_check_cmd_context (cmd, NOT_IN_DIR_LOC_FILE | NOT_IN_LIMIT);
+
+	if (err != NULL) {
+		return err;
+	}
+
+	conf->default_uid = ap_uname2id(uid);
+	conf->default_gid = ap_gname2id(gid);
+
+	return NULL;
+}
+
+
+static const char *set_minuidgid (cmd_parms *cmd, void *mconfig, const char *uid, const char *gid)
+{
+	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
+	const char *err = ap_check_cmd_context (cmd, NOT_IN_DIR_LOC_FILE | NOT_IN_LIMIT);
+
+	if (err != NULL) {
+		return err;
+	}
+
+	conf->min_uid = ap_uname2id(uid);
+	conf->min_gid = ap_gname2id(gid);
+
+	return NULL;
+}
+
+
 static const char *set_documentchroot (cmd_parms *cmd, void *mconfig, const char *chroot_dir, const char *document_root)
 {
 	ruid_config_t *conf = ap_get_module_config (cmd->server->module_config, &ruid2_module);
@@ -257,19 +282,20 @@ static const char *set_documentchroot (cmd_parms *cmd, void *mconfig, const char
 
 	conf->chroot_dir = chroot_dir;
 	conf->document_root = document_root;
-	
+	chroot_used |= RUID_CHROOT_USED;
+
 	return NULL;
 }
-                                                                        
+
 
 /* configure options in httpd.conf */
 static const command_rec ruid_cmds[] = {
 
-	AP_INIT_TAKE1 ("RMode", set_mode, NULL, RSRC_CONF | ACCESS_CONF, "stat or config (default stat)"),
-	AP_INIT_ITERATE ("RGroups", set_groups, NULL, RSRC_CONF | ACCESS_CONF, "Set aditional groups"),
-	AP_INIT_TAKE2 ("RMinUidGid", set_minuidgid, NULL, RSRC_CONF, "Minimal uid or gid file/dir, else set[ug]id to default (RDefaultUidGid)"),
-	AP_INIT_TAKE2 ("RDefaultUidGid", set_defuidgid, NULL, RSRC_CONF, "If uid or gid is < than RMinUidGid set[ug]id to this uid gid"),
+	AP_INIT_TAKE1 ("RMode", set_mode, NULL, RSRC_CONF | ACCESS_CONF, "Set mode to config or stat (default: config)"),
 	AP_INIT_TAKE2 ("RUidGid", set_uidgid, NULL, RSRC_CONF | ACCESS_CONF, "Minimal uid or gid file/dir, else set[ug]id to default (User,Group)"),
+	AP_INIT_ITERATE ("RGroups", set_groups, NULL, RSRC_CONF | ACCESS_CONF, "Set aditional groups"),
+	AP_INIT_TAKE2 ("RDefaultUidGid", set_defuidgid, NULL, RSRC_CONF, "If uid or gid is < than RMinUidGid set[ug]id to this uid gid"),
+	AP_INIT_TAKE2 ("RMinUidGid", set_minuidgid, NULL, RSRC_CONF, "Minimal uid or gid file/dir, else set[ug]id to default (RDefaultUidGid)"),
 	AP_INIT_TAKE2 ("RDocumentChRoot", set_documentchroot, NULL, RSRC_CONF, "Set chroot directory and the document root inside"),
 	{NULL}
 };
@@ -290,10 +316,10 @@ static int ruid_init (apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp, server
 	apr_pool_userdata_get(&data, userdata_key, s->process->pool);
 	if (!data) {
 		apr_pool_userdata_set((const void *)1, userdata_key, apr_pool_cleanup_null, s->process->pool);
-	} else {	                                              
+	} else {
 		ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL, MODULE_NAME "/" MODULE_VERSION " enabled");
 	}
-	
+
 	return OK;
 }
 
@@ -302,12 +328,12 @@ static int ruid_init (apr_pool_t *p, apr_pool_t *plog, apr_pool_t *ptemp, server
 static apr_status_t ruid_child_exit(void *data)
 {
 	int fd = (int)((long)data);
-	
+
 	if (close(fd) < 0) {
 		ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR closing root file descriptor (%d) failed", MODULE_NAME, fd);
 		return APR_EGENERAL;
 	}
-	                                        
+
 	return APR_SUCCESS;
 }
 
@@ -315,60 +341,50 @@ static apr_status_t ruid_child_exit(void *data)
 /* run after child init we are uid User and gid Group */
 static void ruid_child_init (apr_pool_t *p, server_rec *s)
 {
-	ruid_config_t *conf;
 	int ncap;
 	cap_t cap;
 	cap_value_t capval[4];
-	
-	/* detect default uig/gid/groups */
-	/* TODO */	
 
-	/* add module name to signature */
-	// ap_add_version_component(p, MODULE_NAME "/" MODULE_VERSION);
-	
-	stat_mode = UNSET;
-	chroot_root = UNSET;
-	while (s) {
-		conf = ap_get_module_config(s->module_config, &ruid2_module);
-		
-		/* detect stat mode usage */
-		if (conf->stat_used == SET && stat_mode == UNSET) {
-			stat_mode = SET;
-		}
-		
-		/* setup chroot jailbreak */
-		if (conf->chroot_dir && chroot_root == UNSET) {
-			if ((chroot_root = open("/.", O_RDONLY)) < 0) {
-				ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR opening root file descriptor failed (%s)", MODULE_NAME, strerror(errno));
-			} else {
-				/* register cleanup function */
-				apr_pool_cleanup_register(p, (void*)((long)chroot_root), ruid_child_exit, apr_pool_cleanup_null);
-			}
+	/* detect default supplementary group IDs */
+	if ((startup_groupsnr = getgroups(RUID_MAXGROUPS, startup_groups)) == -1) {
+		startup_groupsnr = 0;
+		ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s ERROR getgroups() failed on child init, ignoring supplementary group IDs", MODULE_NAME);
+	}
+
+	/* setup chroot jailbreak */
+	if (chroot_used == RUID_CHROOT_USED) {
+		if ((root_handle = open("/.", O_RDONLY)) < 0) {
+			root_handle = UNSET;
+			ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR opening root file descriptor failed (%s)", MODULE_NAME, strerror(errno));
+		} else {
+			/* register cleanup function */
+			apr_pool_cleanup_register(p, (void*)((long)root_handle), ruid_child_exit, apr_pool_cleanup_null);
 		}
-		
-		if (stat_mode != UNSET && chroot_root != UNSET) {
-			s = NULL;
-		} else {
-	    	        s = s->next;
-	    	}
+	} else {
+		root_handle = UNSET;
 	}
 
-	ncap = 2;
 	/* init cap with all zeros */
 	cap = cap_init();
+
 	capval[0] = CAP_SETUID;
 	capval[1] = CAP_SETGID;
-	if (stat_mode == SET) capval[ncap++] = CAP_DAC_READ_SEARCH;
-	if (chroot_root != UNSET) capval[ncap++] = CAP_SYS_CHROOT;
+	ncap = 2;
+	if (mode_stat_used == RUID_MODE_STAT_USED) {
+		capval[ncap++] = CAP_DAC_READ_SEARCH;
+	}
+	if (root_handle != UNSET) {
+		capval[ncap++] = CAP_SYS_CHROOT;
+	}
 	cap_set_flag(cap, CAP_PERMITTED, ncap, capval, CAP_SET);
 	if (cap_set_proc(cap) != 0) {
-    		ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR %s:cap_set_proc failed", MODULE_NAME, __func__);
+		ap_log_error(APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR %s:cap_set_proc failed", MODULE_NAME, __func__);
 	}
 	cap_free(cap);
 
 	/* MaxRequestsPerChild MUST be 1 to enable drop capability mode */
 	cap_mode = (ap_max_requests_per_child == 1 ? RUID_CAP_MODE_DROP : RUID_CAP_MODE_KEEP);
-		
+
 	/* check if process is dumpable */
 	coredump = prctl(PR_GET_DUMPABLE);
 }
@@ -378,13 +394,13 @@ static void ruid_child_init (apr_pool_t *p, server_rec *s)
 static apr_status_t ruid_suidback (void *data)
 {
 	request_rec *r = data;
-	
+
 	ruid_config_t *conf = ap_get_module_config (r->server->module_config, &ruid2_module);
 	core_server_config *core = (core_server_config *) ap_get_module_config(r->server->module_config, &core_module);
 
 	cap_t cap;
 	cap_value_t capval[3];
-	
+
 	if (cap_mode == RUID_CAP_MODE_KEEP) {
 
 		cap=cap_get_proc();
@@ -397,19 +413,19 @@ static apr_status_t ruid_suidback (void *data)
 		}
 		cap_free(cap);
 
-		setgroups(0,NULL);
-		setgid(unixd_config.group_id);
-		setuid(unixd_config.user_id);
-	
+		setgroups(startup_groupsnr, startup_groups);
+		setgid(ap_unixd_config.group_id);
+		setuid(ap_unixd_config.user_id);
+
 		/* set httpd process dumpable after setuid */
 		if (coredump) {
 			prctl(PR_SET_DUMPABLE,1);
 		}
-		
+
 		/* jail break */
 		if (conf->chroot_dir) {
-			if (fchdir(chroot_root) < 0) {
-				ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s failed to fchdir to root dir (%d) (%s)", MODULE_NAME, chroot_root, strerror(errno));
+			if (fchdir(root_handle) < 0) {
+				ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s failed to fchdir to root dir (%d) (%s)", MODULE_NAME, root_handle, strerror(errno));
 			} else {
 				if (chroot(".") != 0) {
 					ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s jail break failed", MODULE_NAME);
@@ -433,14 +449,16 @@ static apr_status_t ruid_suidback (void *data)
 }
 
 
-static int ruid_set_perm (request_rec *r, const char *from_func) 
+static int ruid_set_perm (request_rec *r, const char *from_func)
 {
 	ruid_config_t *conf = ap_get_module_config(r->server->module_config, &ruid2_module);
 	ruid_dir_config_t *dconf = ap_get_module_config(r->per_dir_config, &ruid2_module);
-	
-	int retval = DECLINED, i;
+
+	int retval = DECLINED;
 	gid_t gid;
 	uid_t uid;
+	gid_t groups[RUID_MAXGROUPS];
+	int groupsnr;
 
 	cap_t cap;
 	cap_value_t capval[4];
@@ -454,54 +472,60 @@ static int ruid_set_perm (request_rec *r, const char *from_func)
 	}
 	cap_free(cap);
 
-	if (dconf->ruid_mode==RUID_MODE_STAT || dconf->ruid_mode==RUID_MODE_UNDEFINED) {
+	if (dconf->ruid_mode==RUID_MODE_STAT) {
 		/* set uid,gid to uid,gid of file
 		 * if file does not exist, finfo.user and finfo.group is set to uid,gid of parent directory
 		 */
 		gid=r->finfo.group;
 		uid=r->finfo.user;
 	} else {
-		gid=dconf->ruid_gid;
-		uid=dconf->ruid_uid;
+		gid=(dconf->ruid_gid == UNSET) ? ap_unixd_config.group_id : dconf->ruid_gid;
+		uid=(dconf->ruid_uid == UNSET) ? ap_unixd_config.user_id : dconf->ruid_uid;
 	}
 
 	/* if uid of filename is less than conf->min_uid then set to conf->default_uid */
-	if (uid < conf->min_uid || uid == UNSET) {
+	if (uid < conf->min_uid) {
 		uid=conf->default_uid;
 	}
-	if (gid < conf->min_gid || gid == UNSET){
+	if (gid < conf->min_gid) {
 		gid=conf->default_gid;
 	}
 
-	if (dconf->groupsnr>0) {
-		for (i=0; i < dconf->groupsnr; i++) {
-			if (dconf->groups[i] < conf->min_gid) {
-				dconf->groups[i]=conf->default_gid;
+	/* set supplementary groups */
+	if ((dconf->groupsnr == UNSET) && (startup_groupsnr > 0)) {
+		memcpy(groups, startup_groups, sizeof(groups));
+		groupsnr = startup_groupsnr;
+	} else if (dconf->groupsnr > 0) {
+		for (groupsnr = 0; groupsnr < dconf->groupsnr; groupsnr++) {
+			if (dconf->groups[groupsnr] >= conf->min_gid) {
+				groups[groupsnr] = dconf->groups[groupsnr];
+			} else {
+				groups[groupsnr] = conf->default_gid;
 			}
-		} 	
-		setgroups(dconf->groupsnr, dconf->groups);
+		}
 	} else {
-		setgroups(0, NULL);
+		groupsnr = 0;
 	}
+	setgroups(groupsnr, groups);
 
 	/* final set[ug]id */
-	if (setgid (gid) != 0)
+	if (setgid(gid) != 0)
 	{
 		ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s %s %s %s>%s:setgid(%d) failed. getgid=%d getuid=%d", MODULE_NAME, ap_get_server_name(r), r->the_request, from_func, __func__, dconf->ruid_gid, getgid(), getuid());
 		retval = HTTP_FORBIDDEN;
 	} else {
-		if (setuid (uid) != 0)
+		if (setuid(uid) != 0)
 		{
 			ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s %s %s %s>%s:setuid(%d) failed. getuid=%d", MODULE_NAME, ap_get_server_name(r), r->the_request, from_func, __func__, dconf->ruid_uid, getuid());
 			retval = HTTP_FORBIDDEN;
 		}
 	}
-	
+
 	/* set httpd process dumpable after setuid */
 	if (coredump) {
 		prctl(PR_SET_DUMPABLE,1);
 	}
-	
+
 	/* clear capabilties from effective set */
 	cap=cap_get_proc();
 	capval[0]=CAP_SETUID;
@@ -514,7 +538,7 @@ static int ruid_set_perm (request_rec *r, const char *from_func)
 		capval[3]=CAP_SYS_CHROOT;
 		cap_set_flag(cap,CAP_PERMITTED,4,capval,CAP_CLEAR);
 	}
-		                            
+
 	if (cap_set_proc(cap)!=0) {
 		ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR %s>%s:cap_set_proc failed after setuid", MODULE_NAME, from_func, __func__);
 	}
@@ -525,10 +549,10 @@ static int ruid_set_perm (request_rec *r, const char *from_func)
 
 
 /* run in post_read_request hook */
-static int ruid_setup (request_rec *r) {
-
-	 /* We decline when we are in a subrequest. The ruid_setup function was
-	  * already executed in the main request. */
+static int ruid_setup (request_rec *r)
+{
+	/* We decline when we are in a subrequest. The ruid_setup function was
+	 * already executed in the main request. */
 	if (!ap_is_initial_req(r)) {
 		return DECLINED;
 	}
@@ -536,13 +560,13 @@ static int ruid_setup (request_rec *r) {
 	ruid_config_t *conf = ap_get_module_config (r->server->module_config,  &ruid2_module);
 	ruid_dir_config_t *dconf = ap_get_module_config(r->per_dir_config, &ruid2_module);
 	core_server_config *core = (core_server_config *) ap_get_module_config(r->server->module_config, &core_module);
-         
+
 	int ncap=0;
 	cap_t cap;
 	cap_value_t capval[2];
 
 	if (dconf->ruid_mode==RUID_MODE_STAT) capval[ncap++] = CAP_DAC_READ_SEARCH;
-	if (chroot_root != UNSET) capval[ncap++] = CAP_SYS_CHROOT;
+	if (root_handle != UNSET) capval[ncap++] = CAP_SYS_CHROOT;
 	if (ncap) {
 		cap=cap_get_proc();
 		cap_set_flag(cap, CAP_EFFECTIVE, ncap, capval, CAP_SET);
@@ -550,8 +574,8 @@ static int ruid_setup (request_rec *r) {
 			ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL, "%s CRITICAL ERROR %s:cap_set_proc failed", MODULE_NAME, __func__);
 		}
 		cap_free(cap);
-	}	
-	
+	}
+
 	/* do chroot trick only if chrootdir is defined */
 	if (conf->chroot_dir)
 	{
@@ -567,7 +591,7 @@ static int ruid_setup (request_rec *r) {
 			ap_log_error (APLOG_MARK, APLOG_ERR, 0, NULL,"%s %s %s chroot to %s failed", MODULE_NAME, ap_get_server_name (r), r->the_request, conf->chroot_dir);
 			return HTTP_FORBIDDEN;
 		}
-	
+
 		cap = cap_get_proc();
 		capval[0] = CAP_SYS_CHROOT;
 		cap_set_flag(cap, CAP_EFFECTIVE, 1, capval, CAP_CLEAR);
@@ -577,10 +601,10 @@ static int ruid_setup (request_rec *r) {
 		}
 		cap_free(cap);
 	}
-	
+
 	/* register suidback function */
 	apr_pool_cleanup_register(r->pool, r, ruid_suidback, apr_pool_cleanup_null);
-		
+
 	if (dconf->ruid_mode==RUID_MODE_CONF)
 	{
 		return ruid_set_perm(r, __func__);
@@ -593,7 +617,7 @@ static int ruid_setup (request_rec *r) {
 /* run in map_to_storage hook */
 static int ruid_uiiii (request_rec *r)
 {
-        return ruid_set_perm(r, __func__);
+		return ruid_set_perm(r, __func__);
 }
 
 
